import os

import matplotlib
matplotlib.use('agg')
from matplotlib import pyplot

import numpy as np
from scipy.optimize import curve_fit
from scipy.integrate import quad

import common
from common import draw_classic_axes


# Data from Einstein's paper
T = [222.4, 262.4, 283.7, 306.4, 331.3, 358.5, 413.0, 479.2, 520.0, 879.7, 1079.7, 1258.0],
c = [0.384, 0.578, 0.683, 0.798, 0.928, 1.069, 1.343, 1.656, 1.833, 2.671, 2.720, 2.781]

def plot_n_e():
    fig, (ax, ax2) = pyplot.subplots(ncols=2, figsize=(10, 5))
    omega = np.linspace(0.1, 2)
    ax.plot(omega, 1/(np.exp(omega) - 1))
    ax.set_ylim(0, top=3)
    ax.set_xlim(left=0)
    ax.set_xlabel(r'$\hbar \omega$')
    ax.set_xticks([0, 1])
    ax.set_xticklabels(['$0$', '$k_B T$'])
    ax.set_ylabel('$n$')
    ax.set_yticks([1, 2])
    ax.set_yticklabels(['$1$', '$2$'])
    draw_classic_axes(ax, xlabeloffset=.2)
    T = np.linspace(0.01, 2)
    ax2.plot(T, 1/2 + 1/(np.exp(1/T)-1))
    ax2.set_ylim(bottom=0)
    ax2.set_xlabel('$k_B T$')
    ax2.set_xticks([0, 1])
    ax2.set_xticklabels(['$0$', r'$\hbar \omega$'])
    ax2.set_ylabel(r"$\bar\varepsilon$")
    ax2.set_yticks([1/2])
    ax2.set_yticklabels([r'$\hbar\omega/2$'])
    draw_classic_axes(ax2, xlabeloffset=.15)
    fig.savefig('bose_einstein.svg')


def c_einstein(T, T_E=1):
    x = T_E / T
    return 3 * x**2 * np.exp(x) / (np.exp(x) - 1)**2


def integrand(y):
    return y**4 * np.exp(y) / (np.exp(y) - 1)**2


@np.vectorize
def c_debye(T, T_D=1):
    x = T / T_D
    return 9 * x**3 * quad(integrand, 0, 1/x)[0]


def plot_einstein():
    T = np.linspace(0.01, 1.5, 500)
    fig, ax = pyplot.subplots()

    ax.plot(T, c_einstein(T)/3)
    ax.fill_between(T, c_einstein(T)/3, 1, alpha=0.5)

    ax.set_ylim(bottom=0, top=1.2)
    ax.set_xlabel('$T$')
    ax.set_ylabel(r'$\omega$')
    ax.set_xticks([1])
    ax.set_xticklabels([r'$\hbar \omega/k_B$'])
    ax.set_yticks([1])
    ax.set_yticklabels(['$k_B$'])
    pyplot.hlines([1], 0, 1.5, linestyles='dashed')
    draw_classic_axes(ax)
    pyplot.savefig('zeropoint.svg')


def plot_debye():
    T = np.linspace(0.01, 1.5, 500)
    fig, ax = pyplot.subplots()

    ax.plot(T, c_einstein(T), label="Einstein model")
    ax.plot(T, c_debye(T), label="Debye model")

    ax.set_ylim(bottom=0, top=3.4)
    ax.set_xlabel('$T$')
    ax.set_ylabel(r'$\omega$')
    ax.set_xticks([1])
    ax.set_xticklabels([r'$\Theta_D$'])
    ax.set_yticks([3])
    ax.set_yticklabels(['$3Nk_B$'])
    ax.legend(loc='lower right')
    pyplot.hlines([3], 0, 1.5, linestyles='dashed')
    draw_classic_axes(ax, xlabeloffset=0.3)
    pyplot.savefig('debye.svg')


def main():
    os.chdir('docs/figures')
    plot_n_e()
    plot_einstein()
    plot_debye()

if __name__ == "__main__":
    main()
